Index: res/res_timing_timerfd.c
===================================================================
--- res/res_timing_timerfd.c	(wersja 322188)
+++ res/res_timing_timerfd.c	(kopia robocza)
@@ -37,6 +37,7 @@
 #include "asterisk/logger.h"
 #include "asterisk/utils.h"
 #include "asterisk/time.h"
+#include "asterisk/poll-compat.h"
 
 static void *timing_funcs_handle;
 
@@ -159,10 +160,33 @@
 
 static void timerfd_timer_ack(int handle, unsigned int quantity)
 {
-	uint64_t expirations;
+	uint64_t expirations = 0;
 	int read_result = 0;
+	int res;
+	struct pollfd pfd;
+	struct itimerspec curr_timer;
 
 	do {
+		if (timerfd_gettime(handle, &curr_timer) != 0) {
+			ast_log(LOG_WARNING, "Failed to read timer values: %s\n", strerror(errno));
+			break;
+		}
+		if (curr_timer.it_value.tv_sec == 0 && curr_timer.it_value.tv_nsec == 0) {
+			ast_log(LOG_WARNING, "Trying to wait on a disarmed timer\n");
+			break;
+		}
+
+		pfd.fd = handle;
+		pfd.events = POLLIN | POLLPRI;
+
+		res = ast_poll(&pfd, 1, 1000);
+		if (res == 0) {
+			ast_log(LOG_WARNING, "Timerfd timer has not expired for 1 second\n");
+			continue;
+		} else if (res != 1 && (errno != EAGAIN && errno != EINTR)) {
+			ast_log(LOG_ERROR, "Read error: %s\n", strerror(errno));
+			break;
+		}
 		read_result = read(handle, &expirations, sizeof(expirations));
 		if (read_result == -1) {
 			if (errno == EINTR || errno == EAGAIN) {
